//-----------------------------------------------------------------------------------
// FILE NAME: FirmwareC8051.c
// TARGET DEVICE: 	C8051Fxxx
// PROGRAMMER DEVICE:	LPC2103
// CREATED ON:          21/08/2017
// CREATED BY: 		JESUS GARCIA BARAJAS
//
// Code to program a C8051Fxxx device through its C2 interface.
//
// This software targets a LPC2103 device, though it could be easily
// ported to any device.
//
// Code assumes the following Port pin connections:
// LPC2103: | C8051Fxxx:
// P0.4    -> C2CK
// P0.5    -> C2D
// VDD     -> VDD
// GND     -> GND
//
// Program includes:
//- FLASH programming routines (Block Write, Block Read, Page Erase,
//  Device Erase,SFR White and SFR Read)
//- Primitive C2 communication routines (Address Write, Address Read,
//  Data Write, Data Read, Reset, and other support routines)
//- Test software for the above routines
//
// Calling sequences for FLASH Programming Routines:
//
// C2 Initialization Procedure:
// 1) Call InitDevice()
// 2) Call InitPI()
//
// Note: All following routines assume the C2 Initialization Procedure
// has been executed prior to their calling
//
// Block Read Procedure:
// 0) Set parameter <FPDAT> provided by InitDevice()
// 1) Set parameter <AddressC> to the target FLASH starting address
// 2) Set parameter <Length> to the number of bytes to read
// 3) Set the pointer <Block> to the location to store the read data
// 4) Call ReadFlashBlock() (will return a ‘0’ if successful,‘1’ otherwise)
//
// Block Write Procedure:
// 0) Set parameter <FPDAT> provided by InitDevice()
// 1) Set parameter <Address> to the target FLASH starting address
// 2) Set parameter <Length> to the number of bytes to write
// 3) Set the pointer <Block> to the location of the data to be written
// 4) Call WriteFlashBlock() (will return a ‘0’ if successful,‘1’ otherwise)
//
// Page Erase Procedure:
// 0) Set parameter <FPDAT> provided by InitDevice()
// 1) Set parameter <NumPage> to a location within the page to be erased
// 2) Call EraseFlashPage() (will return a ‘0’ if successful,‘1’ otherwise)
//
// Device Erase Procedure:
// 0) Set parameter <FPDAT> provided by InitDevice()
// 1) Call EraseDevice() (will return a ‘0’ if successful,‘1’ otherwise)
//
// SFR Write Procedure:
// 0) Set parameter <FPDAT> provided by InitDevice()
// 1) Set parameter <AddressSFR> to the location of the SFR
// 2) Set parameter <NewSFR> to be written
// 3) Call WriteSFR() (will return a ‘0’ if successful,‘1’ otherwise)
//
// SFR Read Procedure:
// 0) Set parameter <FPDAT> provided by InitDevice()
// 1) Set parameter <AddressSFR> to the location of the SFR
// 3) Call ReadSFR() (will return a ‘0’ if successful,‘1’ otherwise)
//
//------------------------------------------------------------------------

/**********************************************************
                  		Header files
 **********************************************************/
//#include <c8051f000.h>    // SFR declarations
#include "lpc21xx.h"
#include "minilib.h"

#include <string.h>
//#include <asm/termios.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/signal.h>
#include <sys/types.h>


/**********************************************************
                       Define SECTION
**********************************************************/

// Times
#define TIMERESET 100
#define TIMEBIT 2

// Macros
#define C2D (1<<5)
#define C2CK (1<<4)
#define C2D_INPUT IO0DIR &=~(C2D);
#define C2D_OUTPUT IO0DIR |=(C2D);
#define C2D_READ ((IO0PIN & (C2D))>>5)
#define C2D_1 IO0SET |=C2D;
#define C2D_0 IO0CLR |=C2D;
#define C2CK_H IO0SET |=C2CK;
#define C2CK_L IO0CLR |=C2CK;

//Register STATUS ADDRESS 
#define FLBussy    (1<<7)
#define EError     (1<<6)
#define InBusy     (1<<1) 
#define OutReady   (1<<0)

// FLASH information
#define FLASH_SIZE 8192          // FLASH size in bytes
#define NUM_PAGES FLASH_SIZE/512 // Number of 512-byte FLASH pages


// UART
#define MAX_CHAR 16

/**********************************************************
                      GLOBAL SECTION
**********************************************************/

/**********************************************************
                      FUNTION SECTION
**********************************************************/

//----------------------------------------------------------
// Primitive C2 Command Routines
//----------------------------------------------------------
// These routines perform the low-level C2 commands
//----------------------------------------------------------

//-----------------------------------------------------------------------------------
// InitClk()
//-----------------------------------------------------------------------------------
//  -Inicialize C2CK
//-----------------------------------------------------------------------------

void InitClk()
{	
	IO0DIR |=(C2CK);
	C2CK_H;
}

//-----------------------------------------------------------------------------------
// PulseClk()
//-----------------------------------------------------------------------------------
// - Strobe C2CK 
//-----------------------------------------------------------------------------

void PulseClk()
{
	C2CK_L;
	_delay_us(TIMEBIT);
	C2CK_H;
	_delay_us(TIMEBIT);
}

//-----------------------------------------------------------------------------------
// Reset()
//-----------------------------------------------------------------------------------
// - Performs a target device reset by pulling the C2CK pin low for >20us
//-----------------------------------------------------------------------------

void Reset()
{
     InitClk();
     C2CK_L
     _delay_us(TIMERESET);
     C2CK_H
	 _delay_us(TIMEBIT);
  }

//-----------------------------------------------------------------------------------
// whiteByte()
//-----------------------------------------------------------------------------------
// Transmits the write byte in C2D output
//-----------------------------------------------------------------------
    
void WriteByte(int byte)
{
     int i,j;
     C2D_OUTPUT;
     for (i=0;i<8;i++)
     {
        
         j=byte&0x01;							 // Shift out 8 bit byte field.LSB-first
         //_printf("j%d=%08x\n",i,j);//DEBUG_INFO
         if (j==1) C2D_1;
		  if (j==0)	C2D_0;
          //  else (_puts("\r\n\nERROR DE ESCRITURA\r\n\n"));//DEBUG_INFO
         PulseClk();
         byte = (byte>>1);
     }
}
//-----------------------------------------------------------------------------------
// AddressWrite()
//-----------------------------------------------------------------------------------
// - Performs a C2 Address register write (writes the <Byte> input to Address register)
//------------------------------------------------------------------------------------

void AddressWrite(int byte)
{
      
      C2D_INPUT;
      PulseClk();							//Start
      C2D_OUTPUT;
      C2D_1; 
	  PulseClk();							// Instruction field (11b -> Address Write)
	  PulseClk();
      WriteByte(byte);
      C2D_INPUT;
      PulseClk();							//Stop
}

//-----------------------------------------------------------------------------------
// ReadByte()
//-----------------------------------------------------------------------------------
// Returns the read byte in C2D input
//-----------------------------------------------------------------------------------

int ReadByte()
{
    int i,j,byte=0x00000000;
    

    for (i=0;i<8;i++)
     {
         PulseClk();
         byte=(byte>>1);					// Shift in 8 bit byte field.LSB-first
		 j=C2D_READ;							
         //_printf("dato%d=%d\n",i,j);//DEBUG_INFO
         if (j==1) byte=byte|0x80;
         //else byte=byte|0x00;//DEBUG_INFO
         
     }
     return byte;
}

//-----------------------------------------------------------------------------------
// AddressRead()
//-----------------------------------------------------------------------------------
// - Performs a C2 Address register read
// - Returns the 8-bit register content
//-----------------------------------------------------------------------------------

int AddressRead()
{
    
     int Address;
     C2D_INPUT
     PulseClk();							//Start
     C2D_OUTPUT
     C2D_0
     PulseClk();							// Instruction field (10b -> Address Read)
     C2D_1
     PulseClk();
     C2D_INPUT
     Address=ReadByte();
     PulseClk();							//Stop
     return Address;
}

//-----------------------------------------------------------------------------------
// Check()
//-----------------------------------------------------------------------------------
// Returns the read bit in C2D input
//-----------------------------------------------------------------------

int Check()
{
      int i=0;
      PulseClk();
      i=C2D_READ;
      
      return i;
}
    
    
//-----------------------------------------------------------------------------------
// DataWrite()
//-----------------------------------------------------------------------------------
// - Performs a C2 Data register write (writes <byte> input to data register)
//-----------------------------------------------------------------------

void DataWrite(int byte)
{
      int j;
      
      C2D_OUTPUT;						
      PulseClk();							//Start
      C2D_1; 
	  PulseClk();							// Instruction field (01b -> Data Write)
	  C2D_0;
	  PulseClk();							// Length field (00b -> 1 byte)
	  PulseClk();
	  PulseClk();
      WriteByte(byte);
      C2D_INPUT;
      
      do {
            j=Check();
        }while (j==0);

      PulseClk();							//Stop
     
}

//-----------------------------------------------------------------------------------
// DataRead()
//-----------------------------------------------------------------------------------
// - Performs a C2 Data register read
// - Returns the 8-bit register content
//-------------------------------------------------------------------------------

int DataRead() 
{    
     int j,byte;
     //InitClk;
     PulseClk();							//Start
     C2D_OUTPUT;
     C2D_0;
     PulseClk();							// Instruction field (00b-> Data Read)
     PulseClk();
     PulseClk();							// Length field (00b -> 1 byte)
     PulseClk();							
    
     C2D_INPUT;
     
      do {
            j=Check();
        }while (j==0);
     
     byte=ReadByte();
     PulseClk();							//Stop
   
     return byte;
}

//-----------------------------------------------------------------------------------
// FLASH Programming Routines (High Level)
//-----------------------------------------------------------------------------------
//
// These high-level routines perform the FLASH Programming Interface (FPI)
// command sequences.
//---------------------------------------------------------------------------


//-----------------------------------------------------------------------------------
// InitDevice()
//-----------------------------------------------------------------------------------
// - Reads the target Devcie ID register and Return FPDAT
//----------------------------------------------------------------------------  
int InitDevice()
{
    int DeviceID=0x00;
    Reset();
    _delay_ms(50); 
    DeviceID=DataRead();
    _printf("ID=%08x\n",DeviceID);//DEBUG_INFO 	
    if (DeviceID==0x0F) return 0x00AD;
    if (DeviceID==0x28) return 0x00AD;
    if (DeviceID==0x18) return 0x00AD;
    if (DeviceID==0x19) return 0x00AD;  
    else return 0x00B4;
}
  
//-----------------------------------------------------------------------------------
// InitPI()
//-----------------------------------------------------------------------------------
// - Initializes the C2 Interface for FLASH programming  
//----------------------------------------------------------------------                       

void InitPI()
{
     Reset();
     AddressWrite(0x02);//Control register (FPCTL) for C2 Data register accesses
     DataWrite(0x02);   //Write the first key code to enable C2 FLASH programming
     DataWrite(0x04);   //Write the second key code to enable C2 FLASH programming
     DataWrite(0x01);	//Write the third key code to enable C2 FLASH programming (Start)
     _delay_ms(40);	    // Delay for at least 20ms to ensure the target is ready for C2 FLASH programming
}

//-----------------------------------------------------------------------------------
// ReadFlashBlock() 
//-----------------------------------------------------------------------------------
// - Reads a block of FLASH memory starting at <AddressC>
// - The size of the block is defined by <Length>
// - Stores the read data at the location targeted by the pointer <*Block>
// - Assumes that FLASH accesses via C2 have been enabled prior to the function call
// - Function call returns a ‘0’ if successful; returns a ‘1’ if unsuccessful
//------------------------------------------------------------------------------

int ReadFlashBlock (int FPDAT, int *Block, int AddressC, int Length)
{
	if (Length>256) return 1;
	int AddressH=AddressC>>8;
	int AddressL=AddressC&0xFF;
    //_printf("AH:%d\n",AddressH);//DEBUG_INFO
    //_printf("AL:%d\n",AddressL);//DEBUG_INFO

    int i,Data,Address,j;
    j=0;
    i=0;
    AddressWrite(FPDAT);						//Select the FLASH Programming Data register
    DataWrite(0x06);							//Send 0x06 Command Read Block
    
    do {									  	//Wait for input acknowledge
        Address=AddressRead()&InBusy;
        //_printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (Address);

    
    do {										//Wait for status information
        Address=AddressRead()&OutReady;
        //_printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (!(Address));
    
    Data=DataRead();							//Checking the command
    //_printf("Elemento 0D=%08x\n",Data);//DEBUG_INFO
    if (Data==0x0D) Data=0;
    else return 1;
    
    DataWrite(AddressH);						//Send address high byte
    
    do {										//Wait for input acknowledge
        Address=AddressRead()&InBusy;
        //_printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (Address);

    DataWrite(AddressL);						//Send address low byte

    do {										//Wait for input acknowledge
        Address=AddressRead()&InBusy;
        //_printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (Address);

    DataWrite(Length);							//Send block size
    
    do {										//Wait for input acknowledge
        Address=AddressRead()&InBusy;
        //_printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (Address);
    
    do {										//Wait for status information
        Address=AddressRead()&OutReady;
        //_printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (!(Address));
    
    Data=DataRead();							//Checking the command
    //_printf("Elemento 0D=%08x\n",Data);//DEBUG_INFO
    if (Data==0x0D) Data=0;
    else return 1;

    for(j=0;j<(Length);j++){

        do {									//Wait for status information
            Address=AddressRead()&OutReady;
            //_printf("Condicion:%08x\n",Address);//DEBUG_INFO
            i++;
            if (i>1000) return 1;
            }while (!(Address)); 

            Block[j]=(DataRead()&0x000000FF);				// Read data from the FPDAT register
         _printf("Byte leido%d=%02x\n",j,Block[j]);//DEBUG_INFO                         
                          }
    return 0;
}
 
//-----------------------------------------------------------------------------------
// WhiteFlashBlock()
//-----------------------------------------------------------------------------------
// - Writes a block of FLASH memory starting at <Address>
// - The size of the block is defined by <Length>
// - Writes the block stored at the location targetted by <*Block>
// - Assumes that FLASH accesses via C2 have been enabled prior to the function call
// - Function call returns a ‘0’ if successful; returns a ‘1’ if unsuccessful
//------------------------------------------------------------------------------------

int WriteFlashBlock(int FPDAT, int *Block, int AddressC, int Length)
{
	if (Length>256) return 1;
	int AddressH=AddressC>>8;
	int AddressL=AddressC&0xFF;
    int Address,Data,i,j;
    i=0;
    AddressWrite(FPDAT);						//Select the FLASH Programming Data register
    DataWrite(0x07);							// 0x07 Command Block Write
    
    do {										//Wait for input acknowledge
        Address=AddressRead()&InBusy;
        //_printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (Address);

    
    
    do {										// Wait for status information
        Address=AddressRead()&OutReady;
        //_printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (!(Address));


    Data=DataRead();							//Checking the command
    //_printf("Elemento 0D=%08x\n",Data);//DEBUG_INFO
    if (Data==0x0D) Data=0;
    else return 1;

    DataWrite(AddressH);						//Send address high byte
    
    do {										//Wait for input acknowledge
        Address=AddressRead()&InBusy;
        //_printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (Address);

    DataWrite(AddressL);						//Send address low byte
    
    do {										//Wait for input acknowledge
        Address=AddressRead()&InBusy;
        //_printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (Address);

    DataWrite(Length);							//Send block size
    
    do {										//Wait for input acknowledge
        Address=AddressRead()&InBusy;
        i++;
        if (i>1000) return 1;
        }while (Address);
    
    for(j=0;j<(Length);j++){
        DataWrite(Block[j]&0x000000FF);					// Write data to the FPDAT register
        _printf("Byte escrito%d=%02x\n",j,Block[j]);
                         
        do {									//Wait for input acknowledge
            Address=AddressRead()&InBusy;
            //_printf("Condicion:%08x\n",Address);//DEBUG_INFO
            i++;
            if (i>1000) return 1;
            }  while (Address);
    }
    
    do {										//Wait for status information
        Address=AddressRead()&OutReady;
        //_printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (!(Address));

    Data=DataRead();							//Checking the command
    //_printf("Elemento 0D=%08x\n",Data);//DEBUG_INFO
    if (Data==0x0D) return 0;
    else return 1; 
}     

//-----------------------------------------------------------------------------------
// EraseFlashPage()
//-----------------------------------------------------------------------------------
// - Erases a 512-byte FLASH page
// - Targets the FLASH page containing the address <NumPage>
// - Assumes that FLASH accesses via C2 have been enabled prior to the function call
// - Function call returns a ‘0’ if successful; returns a ‘1’ if unsuccessful
//------------------------------------------------------------------------------------

int EraseFlashPage ( int FPDAT, int NumPage)
{
    int Address,Data;
    int i=0;
    AddressWrite(FPDAT);					//Select the FLASH Programming Data register
    DataWrite(0x08);						//0x08 command Erase Page
    
    do {									//Wait for input acknowledge
        Address=AddressRead()&InBusy;
       // _printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }  while (Address);

    do {									//Wait for status information
        Address=AddressRead()&OutReady;
        //_printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (!(Address));

    Data=DataRead();						//Checking the command
   // _printf("Elemento 0D=%08x\n",Data);//DEBUG_INFO
    if (Data==0x0D) Data=0;
    else return 1;

    DataWrite(NumPage);						//Send FLASH page number

    do {									//Wait for input acknowledge					
        Address=AddressRead()&InBusy;
       // _printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }  while (Address);

    do {									//Wait for status information
        Address=AddressRead()&OutReady;
       // _printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (!(Address)); 

    Data=DataRead();						//Checking the command
    if (Data==0x0D) Data=0;
    else return 1;   

    DataWrite(0x00);						//Dummy write to initiate erase

    do {									//Wait for input acknowledge
        Address=AddressRead()&InBusy;
       // _printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (Address);

    do {									//Wait for status information
        Address=AddressRead()&OutReady;
       // _printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (!(Address));

    Data=DataRead();						//Checking the command
   // _printf("Elemento 0D=%08x\n",Data);//DEBUG_INFO
    if (Data==0x0D) return 0;
    else return 1; 
}

//-----------------------------------------------------------------------------------
// WriteSFR()
//-----------------------------------------------------------------------------------
// - Write in the SFR register located in the direction <AddressSFR>
// - The new SFR data will be <NewSFR>
// - Assumes that FLASH accesses via C2 have been enabled prior to the function call
// - Function call returns a ‘0’ if successful; returns a ‘1’ if unsuccessful
//-----------------------------------------------------------------------------------

int WriteSFR(int FPDAT, int AddressSFR, int NewSFR)
{
    int Address,Data;
    int i=0;
    AddressWrite(FPDAT);						//Select the FLASH Programming Data register
    DataWrite(0x0A); 							//0x0A command Write SFR
    do {										//Wait for input acknowledge
        Address=AddressRead()&InBusy;
      //  _printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (Address);

    do {										//Wait for status information
        Address=AddressRead()&OutReady;
      //  _printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (!(Address));

    Data=DataRead();							//Checking the command
   // _printf("Elemento 0D=%08x\n",Data);//DEBUG_INFO
    if (Data==0x0D) Data=0;
    else return 1;

    DataWrite(AddressSFR);						//Send Address SFR

    do {										//Wait for input acknowledge
        Address=AddressRead()&InBusy;
      //  _printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (Address);

    DataWrite(0x01);							//Send command Start

    do {										//Wait for input acknowledge
        Address=AddressRead()&InBusy;
       // _printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (Address);

    DataWrite(NewSFR);							//Send New SFR

    do {										//Wait for input acknowledge
        Address=AddressRead()&InBusy;
       // _printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (Address);

    return 0;
}

//-----------------------------------------------------------------------------------
// ReadSFR()
//-----------------------------------------------------------------------------------
// - Read in the SFR register located in the direction <AddressSFR>
// - Assumes that FLASH accesses via C2 have been enabled prior to the function call
// - Function call returns a ‘0’ if successful; returns a ‘1’ if unsuccessful
//-----------------------------------------------------------------------------------

int ReadSFR(int FPDAT, int AddressSFR)
{
    int Address,Data;
    int i=0;
    AddressWrite(FPDAT);						//Select the FLASH Programming Data register
    DataWrite(0x09); 							//0x09 command Read SFR
    
    do {										//Wait for input acknowledge
        Address=AddressRead()&InBusy;
      //  _printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (Address);

    do {										//Wait for status information
        Address=AddressRead()&OutReady;
      //  _printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (!(Address));

    Data=DataRead();							//Checking the command
   // _printf("Elemento 0D=%08x\n",Data);//DEBUG_INFO
    if (Data==0x0D) Data=0;
    else return 1;
    
    DataWrite(AddressSFR);						//Send Address SFR

    do {										//Wait for input acknowledge
        Address=AddressRead()&InBusy;
       // _printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (Address);

    DataWrite(0x01);							//Send command start

    do {										//Wait for input acknowledge
        Address=AddressRead()&InBusy;
      //  _printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (Address);

    do {										//Wait for status information
        Address=AddressRead()&OutReady;
     //   _printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>1000) return 1;
        }while (!(Address));

    Data=DataRead();    
    _printf("Contenido del SFR 0x%02x=%02x\n",AddressSFR,Data);

    return 0;
}

//-----------------------------------------------------------------------------------
// EraseDevice()
//-----------------------------------------------------------------------------------
// - Erases the entire FLASH memory space
// - Assumes that FLASH accesses via C2 have been enabled prior to the function call
// - Function call returns a ‘0’ if successful; returns a ‘1’ if unsuccessful
//-----------------------------------------------------------------------------------

int EraseDevice(int FPDAT)
{
    int Address,Data;
    int i=0;
    AddressWrite(FPDAT);						//Select the FLASH Programming Data register
    DataWrite(0x03); 							//0x03 command erase device

    do {										//Wait for input acknowledge
    Address=AddressRead()&InBusy;
   _printf("Condicion:%08x\n",Address);//DEBUG_INFO
    i++;
    if (i>2000) return 1;
    }while (Address);

    do {										//Wait for status information
        Address=AddressRead()&OutReady;
        _printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>2000) return 1;
        }while (!(Address));

    Data=DataRead();							//Checking the command
    _printf("Elemento 0D=%08x\n",Data);//DEBUG_INFO
    if (Data==0x0D) Data=0;
    else return 1; 

    DataWrite(0xDE);							//Arming sequence command 1

    do {										//Wait for input acknowledge
    Address=AddressRead()&InBusy;
   // _printf("Condicion:%08x\n",Address);//DEBUG_INFO
    i++;
    if (i>2000) return 1;
    }while (Address);

    DataWrite(0xAD);							//Arming sequence command 2

    do {										//Wait for input acknowledge
    Address=AddressRead()&InBusy;
   // _printf("Condicion:%08x\n",Address);//DEBUG_INFO
    i++;
    if (i>2000) return 1;
    }while (Address);

    DataWrite(0xA5);							//Arming sequence command 3

    do {										//Wait for input acknowledge
    Address=AddressRead()&InBusy;
  //  _printf("Condicion:%08x\n",Address);//DEBUG_INFO
    i++;
    if (i>2000) return 1;
    }while (Address);

    do {										//Wait for status information
        Address=AddressRead()&OutReady;
       _printf("Condicion:%08x\n",Address);//DEBUG_INFO
        i++;
        if (i>2000) return 1;
        }while (!(Address));

    Data=DataRead();							//Checking the command
    _printf("Elemento 0D=%08x\n",Data);//DEBUG_INFO
    if (Data==0x0D) return 0;
    else return 1;
}

//-----------------------------------------------------------------------------------
// UART Control Functions
//-----------------------------------------------------------------------------------
// - Allow read and write data in the UART
//-----------------------------------------------------------------------------------
// -------------------------------------------------------
void UART0_putch(char c)
{
 while(!(U0LSR&0x20)); // mientras que U0LSR.THRE == 0 
 U0THR = c;
}
// -------------------------------------------------------
char UART0_getch()
{
 while(!(U0LSR&0x01)); // mientras que U0LSR.DR == 0
 return U0RBR;
}
// -------------------------------------------------------
/*
void UART0_print(char * s)
{
 int i;
 i= 0;
 while(s[i]) UART0_putch(s[i++]);
}
*/
// ---------------------------------------------------------------
/*
void UART0ReadData(char *T)
{
 int i;
 unsigned char c;

 i=0;
 do
 {
  c=UART0_getch();
  UART0_putch(c);
  T[i] = c;
  i++; 
 } while( (c!='\r') && (i<MAX_CHAR) );
 T[i-1]=0;
}
*/
void UART0_Init()
{
 U0LCR = 0x83;  // 10000011 Line Control Register
                //       .. Character length 11 = 8 bits
                //      .   Stop bits         0 = 1 bit
                //     .    Parity enable     0 = No parity
                //    .     Even parity       non sense
                //   .      Stick parity      non sense
                //  .       Break control     0 = No break
                // .        DLAB              1 = Access DLM,DLL
 U0DLL = 32;    
 U0DLM =  0;    // Baud = 4*14745600/(16* 32) = 115200
 U0LCR = 0x03;  // Same a above, but DLAB = 0 = Normal access 
 U0FCR = 0x07;  // 00000111 FIFO Control Register
                //        . FIFO enable       1 = TX&RX FIFO on 
                //       .  RX FIFO Reset     1 = Reset RX FIFO
                //      .   TX FIFO Reset     1 = Reset TX FIFO
                // ..       RX int.t trigger level 00 = 1 byte
};
// -------------------------------------------------------
/**********************************************************
                      MAIN SECTION
**********************************************************/

void main (void)
{
    unsigned int Comand,NewSFR,Address;
    unsigned int SFR=0x00;
    unsigned int i=0,j,z,h,NUM,NumPage;
    unsigned int FPDAT=0x0000;
    unsigned int Aux=0x00000000;

    
    unsigned char BufferRead[256];
    unsigned char *PTR_Read=BufferRead;
/*
    for(j=0;j<(256);j++){
                       
                      BufferRead[j]=0x00; // 
                      
    }
 */   
    //_puts("\r\n\nHello World\r\n\n");//Welcome

    for(j=0;j<6;j++){// j:Number of UART readings
                          
            h=0;         //read UART data
            do{
                BufferRead[h]=UART0_getch();
              // printf("Byte %d:%02x\n",h,BufferRead[h] );//Debug BufferRead  
                h++;
            }while (!((BufferRead[h-1]==0x0A)&&(BufferRead[h-2]==0x0D)));

            //_puts("\r\n\nBuffer de Recepción:\r\n\n");
 /*           for(i=0;i<(30);i++){
                       
                      printf("Byte %d:%02x\n",i,BufferRead[i] );//Debug BufferRead 
                      
            }*/

          

/************************* Menu for selection of the chosen command *****************************/           
            Comand=BufferRead[0];
            switch (Comand)
                    {
                        case 0:{
                            NUM=BufferRead[1];
                            Aux=((BufferRead[2]<<8)&0xFF00);
                            Address=(BufferRead[3]&0x00FF)|Aux;
                            for (i=0;i<NUM;i++){
                                BufferRead[i]=BufferRead[4+i];   
                            }
                            *PTR_Read=BufferRead;
                            PTR_Read=&BufferRead[4];
                            FPDAT=InitDevice();
                            InitPI();
                            z=WriteFlashBlock(FPDAT,PTR_Read,Address,NUM);
                           // if (z==(1)) _puts("\r\nERROR \r\n\n");
                           // if (z==(0)) _puts("\r\nEscritura correcta \r\n\n");
                            break;
                        }
                        case 1:
                            NUM=BufferRead[1];
                            Aux=((BufferRead[2]<<8)&0xFF00);
                            Address=(BufferRead[3]&0x00FF)|Aux;
                            for (i=0;i<NUM;i++){
                                BufferRead[i]=BufferRead[4+i];   
                            }
                            *PTR_Read=BufferRead;
                            FPDAT=InitDevice();
                            InitPI();
                            z=WriteFlashBlock(FPDAT,PTR_Read,Address,NUM);
                          //  if (z==(1)) _puts("\r\nERROR \r\n\n");
                          //  if (z==(0)) _puts("\r\nEscritura correcta \r\n\n");
                            break;
                        case 2:
                            *PTR_Read=BufferRead;
                            NUM=BufferRead[1];
                            Address=BufferRead[2];
                            FPDAT=InitDevice();
                            InitPI();
                            z=ReadFlashBlock(FPDAT,PTR_Read,Address,NUM);
                         //   if (z==1) _puts("\r\nERROR \r\n\n");
                         //   if (z==0) _puts("\r\nLectura correcta \r\n\n");
                            break;
                        
                        case 3:
                            NumPage=BufferRead[1];
                            FPDAT=InitDevice();
                            InitPI();
                            z=EraseFlashPage (FPDAT,NumPage);
                          //  if (z==1) _puts("\r\nERROR \r\n\n");
                          //  if (z==0) _puts("\r\nBorrado por Página correcto \r\n\n");
                            break;
                        case 4:
                            FPDAT=InitDevice();
                            InitPI();
                            z=EraseDevice (FPDAT);
                         //   if (z==1) _puts("\r\nERROR \r\n\n");
                         //   if (z==0) _puts("\r\nBorrado del dispositivo correcto \r\n\n");
                            break;
                        case 5:
                            SFR=BufferRead[1];
                            NewSFR=BufferRead[2];
                            FPDAT=InitDevice();
                            InitPI();
                            z=WriteSFR(FPDAT,SFR,NewSFR);
                         //   if (z==(1)) _puts("\r\nERROR \r\n\n");
                         //   if (z==(0)) _puts("\r\nEscritura de SFR correcta \r\n\n");
                            break;
                        case 6:
                            SFR=BufferRead[1];
                            FPDAT=InitDevice();
                            InitPI();
                            z=ReadSFR(FPDAT,SFR);
                            if (z==(1)) _puts("\r\nERROR \r\n\n");
                            if (z==(0)) _puts("\r\nLectura de SFR correcta \r\n\n");
                            break;
            } //switch     
            UART0_putch(0x0C); //Report to the Pc of completed function successfully 
            //_delay_ms(2000);
            
    }//for
    



/************************* Test software for the above routines  **********************************/



/*
 

    FPDAT=InitDevice();
    InitPI();


   
    j=ReadSFR(FPDAT,0x98);		//SFR SCON0 reset value:0x40 
    if (j==(1)) _puts("\r\nERROR \r\n\n");
    if (j==(0)) _puts("\r\nLectura de SFR correcta  \r\n\n");
 
   	j=WriteSFR(FPDAT,0x98,0xC8);   //SFR SCON0 new value 0xC8 9-bit UART with Variable Baud Rate
    if (j==(1)) _puts("\r\nERROR \r\n\n");
	if (j==(0)) _puts("\r\nEscritura de SFR correcta \r\n\n");

     j=ReadSFR(FPDAT,0x98);						 
    if (j==(1)) _puts("\r\nERROR \r\n\n");
    if (j==(0)) _puts("\r\nLectura de SFR correcta  \r\n\n");
    _delay_ms(1000);
 

    j=EraseDevice(FPDAT);
    if (j==1) _puts("\r\nERROR \r\n\n");
    if (j==0) _puts("\r\nBorrado del dispositivo correcto\r\n\n");
    
    _delay_ms(4000); 
    j=EraseFlashPage (FPDAT,0);
    if (j==1) _puts("\r\nERROR \r\n\n");
    if (j==0) _puts("\r\nBorrado por Página correcto \r\n\n");
    _delay_ms(4000); 
    j=WriteFlashBlock(FPDAT,PTR_Read,0x0100,50);
	if (j==(1)) _puts("\r\nERROR \r\n\n");
	if (j==(0)) _puts("\r\nEscritura correcta \r\n\n");
    _delay_ms(400); 
    j=ReadFlashBlock(FPDAT,PTR_Read,0x00FF,256);
    if (j==1) _puts("\r\nERROR \r\n\n");
    if (j==0) _puts("\r\nLectura correcta \r\n\n");
    _delay_ms(400); 

    for(i=0;i<(256);i++){
                          
              _printf("Elemento%d=%08x\n",i,BufferRead[i]);//DEBUG_INFO

                      }


*/
	
	while (1);//PCON=1;		// CPU idle

}

